//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//
 
//Unix implementation

#include <fcntl.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdio.h>

#include "definitions.h"
#include "devio.h"
#include "f11asrt.h"
#include "files11.h"

int devio_open_dev(const char* dev_name, int mode)
{
    errno = 0;
    return (int)open(dev_name, 
		     ((mode & DEVIO_RO) ? O_RDONLY : O_RDWR), 
		     0666);
}

void devio_close_dev(int dev_handle)
{
    errno = 0;
    close(dev_handle);
}

int devio_read_dev(int dev_handle, int blk_number, int blk_count,
		   void* buffer)
{
    off_t offset = blk_number * F11_BLOCK_SIZE;
    int length = blk_count * F11_BLOCK_SIZE;

    errno = 0;
    ASSERT(length);
    ASSERT(buffer);

    TRACE(("devio_read_dev: offset = %lx, len = %x\n", offset, length));
    if (lseek(dev_handle, offset, SEEK_SET) == (off_t)-1 ) {
	TRACE(("devio_read_dev: lseek failed!\n"));
	return -1;
    }

    return read(dev_handle, buffer, length);
}

int devio_write_dev(int dev_handle, int blk_number, int blk_count,
		    void* buffer)
{
    off_t offset = blk_number * F11_BLOCK_SIZE;
    int length = blk_count * F11_BLOCK_SIZE;

    errno = 0;
    ASSERT(length);
    ASSERT(buffer);

    TRACE(("devio_write_dev: offset = %lx, len = %x\n", offset, length));
    if (lseek(dev_handle, offset, SEEK_SET) == (off_t)-1 ) {
	TRACE(("devio_write_dev: lseek failed!\n"));
	return -1;
    }

    return write(dev_handle, buffer, length);
}

int devio_dev_error(void)
{
    return errno;
}

char* devio_dev_error_msg(void)
{
    return strerror(errno);
}

