//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 Structure level 2 

#include <stdlib.h>
#include <string.h>

#include "definitions.h"
#include "f11asrt.h"
#include "devio.h"
#include "files11.h"
#include "f11liberr.h"
#include "f11file.h"
#include "f11file_dir.h"
#include "f11vol.h"
#include "f11file_sys.h"

static const ODS_FID	idx_fid(FID_INDEXF_NUMB, FID_INDEXF_SEQ, FID_RVN_INVOLCONT);
static const ODS_FID	mfd_fid(FID_MFD_NUMB,    FID_MFD_SEQ,    FID_RVN_INVOLCONT);
static const ODS_FID	bmp_fid(FID_BITMAP_NUMB, FID_BITMAP_SEQ, FID_RVN_INVOLCONT);

lbn_t f11_volume_acp::find_and_load_hmblk()
{
    lbn_t	hb_lbn;
    dword_t	delta, i;

    // Search geometry independent
    delta = GEOM_INDEPEND_DELTA;
    for (i = 0; i < GEOM_INDEPEND_NUMB; i++) { 
	hb_lbn = i * delta + HOME_BLK_LBN;

	// read block
	// logical read - OK
	if (f11_read_logical_blk(hb_lbn, 1, &hblk) != F11_BLOCK_SIZE)
	    return INVALID_LBN;

	if (!hblk.is_valid())
	    continue;
	else
	    return hb_lbn;
    }

    return INVALID_LBN;
}

lbn_t f11_volume_acp::get_hdr_lbn(dword_t fid_num)
{
    lbn_t	lbn = INVALID_LBN;
    ODS_FH2*	fh;
    ODS_FM2*	rp;
    dword_t	j = 0, vbn;
    file_hdr*	f;

    ASSERT((fid_num & 0xFF000000) == 0L);

    //  file number is valid ?
    if ((fid_num % 0xFFFF) == 0) {
	set_error(F11_INVALID_FIDX);
	return INVALID_LBN;
    }
    if (fid_num > hblk.hm2_l_maxfiles) {
	set_error(F11_INVALID_FIDX);
	return INVALID_LBN;
    }

    //  calculate fidx lbn
    if (fid_num <= hblk.hm2_w_resfiles) {
	//  IDX or MFD, or other reserved file...
	//  possible to calculate hdr lbn from the hmblk info.
	lbn = hblk.hm2_l_ibmaplbn + 
	    hblk.hm2_w_ibmapsize + 
	    fid_num - 1;

    } else {
	//  Any file use file index file LBN(s) for accessing
	//  file header
	ASSERT(p_indexf_sys);

	//  go through index file lbn(s)
	f = p_indexf_sys->file_header;
	vbn = fid_num - 1 + hblk.hm2_w_ibmapsize +
	    4 * hblk.hm2_w_cluster;

	do {
	    fh = &f->fhdr;
	    rp = fh->get_mp_area();

	    rp_iterator next_rp(rp, p_indexf_sys->get_fhdr()->fh2_b_map_inuse);
	    while ((rp = next_rp++) != NULL)
	    {
		//  handle placement header
		if(rp->rp_isfmt0()) {
		    continue;
		}

		//  found ?
		j += rp->rp_getcount();
		if (j > vbn) {
		    lbn = rp->rp_getlbn() + vbn - (j - rp->rp_getcount());
		    break;
		}
	    }

	    f = f->next;
	} while(f && lbn != INVALID_LBN);

	TRACE(("fid2lbn: vbn 0%d --> lbn 0%d, fid_num 0%d\n", 
	       vbn, lbn, fid_num));

	if(lbn == INVALID_LBN) {
	    set_error(F11_INVALID_FIDX);
	    return INVALID_LBN;
	}
    }

    return lbn;
}

int f11_volume::f11_assign(int channel, int ro_mode)
{
    if (this->chan != INVALID_CHAN) { 
	set_error(F11_DEVICE_OPENED);
	return INVALID_CHAN;
    }
    if (channel == INVALID_CHAN) { 
	set_error(F11_INVALID_ARG);
	return INVALID_CHAN;
    }

    this->mode = ro_mode;
    this->chan = channel;

    return (this->chan = channel);
}

int f11_volume::f11_dassgn()
{
    if (chan == INVALID_CHAN) { 
	set_error(F11_INVALID_HANDLE); 
	return -1; 
    }

    if(chan != INVALID_CHAN)
	devio_close_dev(chan);

    chan = INVALID_CHAN;

    return 0;
}

void f11_volume::f11_sync()
{
    return;
}


int f11_volume_acp::f11_assign(int channel, int ro_mode)
{
    int	sts = SS_NORMAL;

    if (f11_volume::f11_assign(channel, ro_mode) == INVALID_CHAN)
	return INVALID_CHAN;

    // search and read home block
    if(find_and_load_hmblk() == INVALID_LBN) {
	set_error(F11_HB_NOT_FOUND);
	return (this->chan = INVALID_CHAN);
    }

    // open file index file
    p_indexf_sys = new f11_file_indexf_sys;

    // wsize == -1: map the entire file
    sts = acp_sub_access(p_indexf_sys, -1, idx_fid);

    if (sts != SS_NORMAL) 
    {
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	return (this->chan = INVALID_CHAN);
    }

    if(!p_indexf_sys->is_valid(&mfd_fid)) 
    {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	return (this->chan = INVALID_CHAN);
    }

    p_indexf_sys->init_bmap();

    // open mfd file
    p_mfd_dir = new f11_directory;

    // wsize == -1: map the entire file
    sts = acp_sub_access(p_mfd_dir, -1, mfd_fid);

    if (sts != SS_NORMAL) 
    {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	delete p_mfd_dir;
	p_mfd_dir = NULL;
	return (this->chan = INVALID_CHAN);
    }

    if(!p_mfd_dir->is_valid(&mfd_fid)) 
    {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	qio_deaccess(p_mfd_dir, mfd_fid);
	delete p_mfd_dir;
	p_mfd_dir = NULL;
	return (this->chan = INVALID_CHAN);
    }

    // open bitmap file
    p_bitmap_sys = new f11_file_bitmap_sys;

    // wsize == -1: map the entire file
    sts = acp_sub_access(p_bitmap_sys, -1, bmp_fid);

    if (sts != SS_NORMAL) 
    {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	qio_deaccess(p_mfd_dir, mfd_fid);
	delete p_mfd_dir;
	p_mfd_dir = NULL;
	delete p_bitmap_sys;
	p_bitmap_sys = NULL;
	return (this->chan = INVALID_CHAN);
    }

    if(!p_bitmap_sys->is_valid(&mfd_fid)) 
    {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
	qio_deaccess(p_mfd_dir, mfd_fid);
	delete p_mfd_dir;
	p_mfd_dir = NULL;
	qio_deaccess(p_bitmap_sys, bmp_fid);
	delete p_bitmap_sys;
	p_bitmap_sys = NULL;
	return (this->chan = INVALID_CHAN);
    }

    p_bitmap_sys->init_bmap();

    return (this->chan = channel);
}

int f11_volume_acp::f11_dassgn()
{
    if (chan == INVALID_CHAN) { 
	set_error(F11_INVALID_HANDLE); 
	return -1; 
    }

    ASSERT(p_indexf_sys && p_mfd_dir && p_bitmap_sys);

    this->f11_sync();

    // close fidx, mfd, current directory, current file
    if(p_indexf_sys) {
	qio_deaccess(p_indexf_sys, idx_fid);
	delete p_indexf_sys;
	p_indexf_sys = NULL;
    }

    if(p_mfd_dir) {
	qio_deaccess(p_mfd_dir, mfd_fid);
	delete p_mfd_dir;
	p_mfd_dir = NULL;
    }

    if(p_bitmap_sys) {
	qio_deaccess(p_bitmap_sys, bmp_fid);
	delete p_bitmap_sys;
	p_bitmap_sys = NULL;
    }

    return f11_volume::f11_dassgn();
}

void f11_volume_acp::f11_sync()
{
    if(p_indexf_sys) {
	p_indexf_sys->f11_sync();
    }

    if(p_mfd_dir) {
	p_mfd_dir->f11_sync();
    }

    if(p_bitmap_sys) {
	p_bitmap_sys->f11_sync();
    }

    f11_volume::f11_sync();
}


int f11_volume::f11_read_logical_blk(lbn_t lbn, 
	dword_t blkcount, void* buf)
{
    dword_t j;
    j = devio_read_dev(chan, lbn, blkcount, buf);
    if (j != blkcount * F11_BLOCK_SIZE) {
	if (get_dev_error()) {
	    set_error(F11_DRIVE_ERR);
	    return -1;
	}
	set_error(F11_EOF);
    }
    return j;
}


int f11_volume::f11_write_logical_blk(lbn_t lbn, 
	dword_t blkcount, void* buf)
{
    if (mode == DEVIO_RO) {
	set_error(F11_DEVICE_READ_ONLY);
	return -1;
    }

    dword_t j;
    j = devio_write_dev(chan, lbn, blkcount, buf);
    if (j != blkcount * F11_BLOCK_SIZE) {
	if (get_dev_error()) {
		set_error(F11_DRIVE_ERR);
		return -1;
	}
	set_error(F11_EOF);
    }
    return j;
}

