//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 Structure level 2 

#ifndef __F11VOL_H__
#define __F11VOL_H__

#include "devio.h"
#include "files11.h"
#include "f11file.h"

// Forward Declarations
class f11_file;
class f11_file_sequential;
class f11_file_indexf_sys;
class f11_directory;
class f11_file_bitmap_sys;

// Mounted volume, possibly FOREIGN
class f11_volume 
{
public:
    int	    chan;   // Device channel
    int	    mode;   // Read/Only, Write/Only, etc

private:
    int	    lib_error;

public:
    f11_volume() : chan(INVALID_CHAN), mode(0) { 
	lib_error = F11_NO_ERROR;
    }
    
    ~f11_volume() {}

    virtual int	    f11_assign(int chan, int mode);
    virtual int	    f11_dassgn();
    virtual void    f11_sync();

    int	f11_read_logical_blk(lbn_t lbn, dword_t blkcount, void* buf);
    int	f11_write_logical_blk(lbn_t lbn, dword_t blkcount, void* buf);

    void    set_error(int err) { 
	lib_error = err;
    }

    int	    get_dev_error() { 
	return devio_dev_error();
    }

    int	    get_error() { 
	return lib_error;
    }

};

// Mounted Files-11 ODS-2 Volume
class f11_volume_acp : public f11_volume 
{
public:
    ODS_HM2	hblk;				// Device Home Block

    f11_file_indexf_sys*    p_indexf_sys;	// Index file
    f11_directory*	    p_mfd_dir;		// Master File Directory file
    f11_file_bitmap_sys*    p_bitmap_sys;	// Storage Bitmap File

private:
    lbn_t 	get_hdr_lbn(dword_t fid_num);
    lbn_t	find_and_load_hmblk();

public:
    f11_volume_acp() : p_indexf_sys(NULL), 
		       p_mfd_dir(NULL), 
		       p_bitmap_sys(NULL)
    {}

    ~f11_volume_acp() {}

    virtual int		f11_assign(int chan, int mode);
    virtual int		f11_dassgn();
    virtual void	f11_sync();

    // ACP-QIO Major Functions
    int	qio_create(
	    f11_file*	    file,
	    dword_t	    io_m_create,
	    dword_t	    io_m_access,
	    dword_t	    io_m_delete,
	    ODS_FID&	    fib_w_fid,
	    const ODS_FID&  fib_w_did,
	    word_t	    fib_w_nmctl_fib_v_newver,
	    word_t	    fib_w_nmctl_fib_v_supersede,
			
	    // for sub dirlookup:
	    word_t	    fib_w_nmctl_fib_v_wild,
	    dword_t&	    fib_l_wcc,
	    byte_t	    fib_b_wsize,

	    // for extend
	    word_t	    fib_w_exctl_fib_v_extend,
	    word_t	    fib_w_exctl_fib_v_alcon,
	    word_t	    fib_w_exctl_fib_v_alconb,
	    word_t	    fib_w_exctl_fib_v_filcon,
	    word_t	    fib_w_exctl_fib_v_aldef,
	    dword_t	    fib_l_exsz,
	    dword_t&	    fib_l_exvbn,

	    const char*	    p2,
	    word_t*	    p3,
	    char*	    p4 );

    int	qio_access(
	    f11_file*	    file,
	    dword_t	    io_m_create,
	    dword_t	    io_m_access,

	    // For create
	    word_t	    fib_w_nmctl_fib_v_newver,
	    word_t	    fib_w_nmctl_fib_v_supersede,

	    // For sub dirlookup:
	    word_t	    fib_w_nmctl_fib_v_wild,
	    ODS_FID&	    fib_w_fid,
	    const ODS_FID&  fib_w_did,
	    dword_t&	    fib_l_wcc,
	    byte_t	    fib_b_wsize,

	    // For extend
	    word_t	    fib_w_exctl_fib_v_extend,
	    word_t	    fib_w_exctl_fib_v_alcon,
	    word_t	    fib_w_exctl_fib_v_alconb,
	    word_t	    fib_w_exctl_fib_v_filcon,
	    word_t	    fib_w_exctl_fib_v_aldef,
	    dword_t	    fib_l_exsz,
	    dword_t&	    fib_l_exvbn,

	    const char*	    p2,
	    word_t*	    p3,
	    char*	    p4 );

    int	qio_deaccess(
	    f11_file*	    f,
	    const ODS_FID&  fib_w_fid );

    int	qio_delete(
	    dword_t	    io_m_delete,

	    // For sub dirlookup:
	    word_t	    fib_w_nmctl_fib_v_wild,

	    ODS_FID&	    fib_w_fid,
	    const ODS_FID&  fib_w_did,
	    dword_t&	    fib_l_wcc,

	    const char*	    p2,
	    word_t*	    p3,
	    char*	    p4 );


    // ACP-QIO SubFunctions
    int	acp_sub_dirlookup(
	    word_t	    fib_w_nmctl_fib_v_wild,
	    ODS_FID&	    fib_w_fid,
	    const ODS_FID&  fib_w_did,
	    dword_t&	    fib_l_wcc,

	    const char*	    p2,
	    word_t*	    p3,
	    char*	    p4 );

    int	acp_sub_access(
	    f11_file*	    file,
	    byte_t	    fib_b_wsize,
	    const ODS_FID&  fib_w_fid );

    int	acp_sub_extend(
	    f11_file* file,
	    word_t	fib_w_exctl_fib_v_extend,
	    word_t	fib_w_exctl_fib_v_alcon,
	    word_t	fib_w_exctl_fib_v_alconb,
	    word_t	fib_w_exctl_fib_v_filcon,
	    word_t	fib_w_exctl_fib_v_aldef,
	    dword_t	fib_l_exsz,
	    dword_t&	fib_l_exvbn );

    int	acp_sub_truncate(
	    f11_file*	file,
	    dword_t&	fib_l_exsz,
	    dword_t	fib_l_exvbn );

    int	acp_sub_create_file(
	    ODS_FID&	fib_w_fid,
	    const char*	p2 );

    int	acp_sub_create_direntry(
	    const ODS_FID&  fib_w_did,
	    const char*	    p2,
	    const ODS_FID&  fib_w_fid,
	    word_t	    version,
	    word_t*	    p3,
	    char*	    p4 );

    int	acp_sub_create_dirrecord(
	    const ODS_FID&  fib_w_did,
	    const char*	    p2,
	    const ODS_FID&  fib_w_fid,
	    word_t	    version,
	    word_t*	    p3,
	    char*	    p4 );
};

#endif 	// __F11VOL_H__
