//
// Copyright (C) 2000 Software Resources International SA
//
// This code is provided on an 'as is' basis.
// Software Resources International SA carries no responsibility for
// the application of this code in any way or form.
//

// Files-11 Structure level 2 

#ifndef __FILES11_H__
#define __FILES11_H__

#include "definitions.h"

#define F11_BLOCK_SIZE		512


// 2.3.2 File Identifier
struct ODS_FID {
    ods_word_t	fid_w_num;	// [0] File number
    ods_word_t	fid_w_seq;	// [2] File sequence number
    byte_t	fid_b_rvn;	// [4] Relative volume number
    byte_t	fid_b_nmx;	// [5] File number extension

public:

    ODS_FID(dword_t numb = 0, word_t seq = 0, byte_t rvn = 0);

    dword_t numb() const {
	return ((dword_t)fid_w_num | ((dword_t)fid_b_nmx << 16));
    }

    int	is_valid() const {
	return (fid_w_num != 0);
    }
    
    bool    operator==(const ODS_FID& fid) const {
	return ((this == &fid) ||
	       ((fid_w_num == fid.fid_w_num) &&
		(fid_w_seq == fid.fid_w_seq) &&
		(fid_b_nmx == fid.fid_b_nmx) &&
		(fid_b_rvn == fid.fid_b_rvn)));
    }
};

// Reserved Files
#define FID_INVALID_NUMB    0
#define FID_INDEXF_NUMB	    1	    // Index File		1,1 INDEXF.SYS;1
#define FID_INDEXF_SEQ	    1
#define FID_BITMAP_NUMB	    2	    // Storage Bitmap File	2,2 BITMAP.SYS;1
#define FID_BITMAP_SEQ	    2
#define FID_BADBLK_NUMB	    3	    // Bad Block File		3,3 BADBLK.SYS;1
#define FID_BADBLK_SEQ	    3
#define FID_MFD_NUMB	    4	    // Master File Directory	4,4 000000.DIR;1
#define FID_MFD_SEQ	    4
#define FID_CORIMG_NUMB	    5	    // System Core Image File	5,5 CORIMG.SYS;1
#define FID_CORIMG_SEQ	    5
#define FID_VOLSET_NUMB	    6	    // Volume Set List File	6,6 VOLSET.SYS;1
#define FID_VOLSET_SEQ	    6
#define FID_CONTIN_NUMB	    7	    // Standard Contin. File	7,7 CONTIN.SYS;1
#define FID_CONTIN_SEQ	    7
#define FID_BACKUP_NUMB	    8	    // Backup File		8,8 BACKUP.SYS;1
#define FID_BACKUP_SEQ	    8
#define FID_BADLOG_NUMB	    9	    // Bad Block Log File	9,9 BADLOG.SYS;1
#define FID_BADLOG_SEQ	    9
#define FID_LASTRESERVED_NUMB	FID_BADLOG_NUMB

#define FID_RVN_INVOLCONT   0	    // FID RVN in context of this volume
#define FID_RVN_NOT_INSET   0	    // FID RVN for volume in not vol set

// Index file && volume struct defs
#define BOOT_BLK_LBN	    0
#define HOME_BLK_LBN	    1

#define BOOT_BLK_CLR	    0
#define HOME_BLK_CLR	    1
#define ALHOME_BLK_CLR	    2
#define ALFIDX_BLK_CLR	    3

// Home block defs
#define GEOM_INDEPEND_DELTA 1033
#define GEOM_INDEPEND_NUMB  10
#define CHECKSUMM1_WORDS    29	// actual + 1
#define CHECKSUMM_WORDS	    255	// actual + 1

#define F11_MAX_FILES	    (0x01000000 - 1)
#define F11_MIN_RESFILES    5
#define FORMAT_STRING	    "DECFILE11B  "
#define DEF_STRUCT_LEV	    2

//  Structure level
struct struct_lev {
    byte_t	ver;	// Version
    byte_t	lev;	// Level
};

//  UIC
struct uic {
    ods_word_t	member;	// Member identifier
    ods_word_t	group;	// Group identifier
};

//  File protection
struct file_prot {		// For every field:
#if !defined(__sparc)
    byte_t	system : 4;	// Bit<0> set to deny read access,
    byte_t	owner  : 4;	// Bit<1> set to deny write access,

    byte_t	group  : 4;	// Bit<2> set to deny exec access,
    byte_t	world  : 4;	// Bit<3> set to deny delete access
#else
    byte_t	owner  : 4;	// Bit<1> set to deny write access,
    byte_t	system : 4;	// Bit<0> set to deny read access,

    byte_t	world  : 4;	// Bit<3> set to deny delete access
    byte_t	group  : 4;	// Bit<2> set to deny exec access,
#endif
};

// 2.5.1.2 Home Block structure
struct ODS_HM2 {
    ods_dword_t	hm2_l_homelbn;	    // [0] home block LBN
				    //   copy of home block
    ods_dword_t	hm2_l_alhomelbn;    // [4] alternate home block LBN,
				    //   equal homelbn for last (or primary)
    ods_dword_t	hm2_l_altidxlbn;    // [8] backup index file header LBN
    struct_lev	hm2_w_struclev;	    // [12] structure level and version
    ods_word_t	hm2_w_cluster;	    // [14] storage bitmap cluster factor
    ods_word_t	hm2_w_homevbn;	    // [16] home block VBN
    ods_word_t	hm2_w_alhomevbn;    // [18] backup home block VBN
    ods_word_t	hm2_w_altidxvbn;    // [20] backup index file header VBN
    ods_word_t	hm2_w_ibmapvbn;	    // [22] index file bitmap VBN
    ods_dword_t	hm2_l_ibmaplbn;	    // [24] index file bitmap LBN
    ods_dword_t	hm2_l_maxfiles;	    // [28] maximum number of files
    ods_word_t	hm2_w_ibmapsize;    // [32] index bitmap size
    ods_word_t	hm2_w_resfiles;	    // [34] number of reserved files
    ods_word_t	hm2_w_devtype;	    // [36] disk dev type
    ods_word_t	hm2_w_rvn;	    // [38] relative volume number
    ods_word_t	hm2_w_setcount;	    // [40] volume number in set
    ods_word_t	hm2_w_volchar;	    // [42] volume characteristics
    uic		hm2_l_volowner;	    // [44] volume ownwer
    ods_dword_t	hm2_pad0;	    // [48] reserved
    ods_word_t	hm2_w_protect;	    // [52] volume protection code
    file_prot	hm2_w_fileprot;	    // [54] default file protection
    ods_word_t	hm2_pad1;	    // [56] default file record protection
    ods_word_t	hm2_w_checksum1;    // [58] first checksumm, additing
				    //   checksumm of all preseding enties
    ods_word_t	hm2_q_credate[4];   // [60] create date	
    byte_t	hm2_b_window;	    // [68] def window size
    byte_t	hm2_b_lru_lim;	    // [69] directory peaccess limit
    ods_word_t	hm2_w_extend;	    // [70] default file extend (blk)
    ods_word_t	hm2_q_retainmin[4]; // [72]	min file retension period
    ods_word_t	hm2_q_retainmax[4]; // [80] max file retension period
    ods_word_t	hm2_q_revdate[4];   // [88] volume revision date
    byte_t	hm2_r_min_class[20];// [96] min security class
    byte_t	hm2_r_max_class[20];// [116] max security class

    byte_t	hm2_pad2[320];	    // [136] reserved

    ods_dword_t	hm2_l_serialnum;    // [456] media serial number
    byte_t	hm2_t_strucname[12];// [460] structure name
    byte_t	hm2_t_volname[12];  // [472] volume name
    byte_t	hm2_t_ownername[12];// [484] volume owner name
    byte_t	hm2_t_format[12];   // [496] "DECFILE11B  " for SL-2

    ods_word_t	hm2_pad3;	    // [508] reserverd
    ods_word_t	hm2_w_checksum2;    // [510] checksum, additing checksumm
				    //   of the preceding 255 words
public:
    int		is_valid() const;

    int		has_valid_checksum() const;
    void	update_checksum();

};

// 2.3.3 File Header structure

//  file characteristics
struct ODS_FCH {
    union {
	struct {
#if !defined(__sparc)
	    byte_t	fch_v_wcontig	    : 1;    // - file was contiguous
	    byte_t	fch_v_nobackup	    : 1;    // + file was not copied by BACKUP util
	    byte_t	fch_v_writeback	    : 1;    // - write back cache may be used
	    byte_t	fch_v_readcheck	    : 1;    // - read check op are to be performed
	    byte_t	fch_v_writecheck    : 1;    // - write check op are to be performed
	    byte_t	fch_v_contigb	    : 1;    // + space is to be allocated cont
	    byte_t	fch_v_locked	    : 1;    // - locked on deaccess
	    byte_t	fch_v_contig	    : 1;    // + file is logicaly contiguous
#else
	    byte_t	fch_v_contig	    : 1;    // + file is logicaly contiguous
	    byte_t	fch_v_locked	    : 1;    // - locked on deaccess
	    byte_t	fch_v_contigb	    : 1;    // + space is to be allocated cont
	    byte_t	fch_v_writecheck    : 1;    // - write check op are to be performed
	    byte_t	fch_v_readcheck	    : 1;    // - read check op are to be performed
	    byte_t	fch_v_writeback	    : 1;    // - write back cache may be used
	    byte_t	fch_v_nobackup	    : 1;    // + file was not copied by BACKUP util
	    byte_t	fch_v_wcontig	    : 1;    // - file was contiguous
#endif
#if !defined(__sparc)
	    byte_t	fch_pad0	    : 3;    // - reserved
	    byte_t	fch_v_badacl	    : 1;    // - ACL is invalid
	    byte_t	fch_v_spool	    : 1;    // - spool file
	    byte_t	fch_v_directory	    : 1;    // + directory file
	    byte_t	fch_v_badblock	    : 1;    // - file has bad blocks
	    byte_t	fch_v_markdel	    : 1;    // + file is marked for deletion
#else
	    byte_t	fch_v_markdel	    : 1;    // + file is marked for deletion
	    byte_t	fch_v_badblock	    : 1;    // - file has bad blocks
	    byte_t	fch_v_directory	    : 1;    // + directory file
	    byte_t	fch_v_spool	    : 1;    // - spool file
	    byte_t	fch_v_badacl	    : 1;    // - ACL is invalid
	    byte_t	fch_pad0	    : 3;    // - reserved
#endif
#if !defined(__sparc)
	    byte_t	fch_v_nocharge	    : 1;    // - file sp isn't to charged to its own
	    byte_t	fch_v_erase	    : 1;    // + file is to be erased on del
	    byte_t	pad1		    : 6;    // - reserved
#else
	    byte_t	pad1		    : 6;    // - reserved
	    byte_t	fch_v_erase	    : 1;    // + file is to be erased on del
	    byte_t	fch_v_nocharge	    : 1;    // - file sp isn't to charged to its own
#endif
	    byte_t	pad2		    : 8;    // - reserved
	} s;
	byte_t	fchar[4];			    // as dword
    } u;
};


//  ACP QIO Record Attributes Area file attributes
struct ACP_FAT {

    // Record Types
    enum ods_fat_rtype {
	fat_c_undefined	= 0,	// undefined (stream binary)
	fat_c_fixed	= 1,	// fixed length
	fat_c_variable	= 2,	// variable length
	fat_c_vfc	= 3,	// variable with fixed control
	fat_c_stream	= 4,	// RMS stream
	fat_c_streamlf	= 5,	// LF-terminated stream format
	fat_c_streamcr	= 6	// CR-terminated stream format
    };

    // File Organization
    enum ods_fat_fileorg {
	fat_c_sequental	= 0,	// sequental file organization
	fat_c_relative	= 1,	// relative file organization
	fat_c_indexed	= 2,	// indexed file organization
	fat_c_direct	= 3	// direct file organization
    };

    struct {
#if !defined(__sparc)
	byte_t	fat_v_rtype   : 4;	// [0] record type
	byte_t	fat_v_fileorg : 4;	// file organization
#else
	byte_t	fat_v_fileorg : 4;	// file organization
	byte_t	fat_v_rtype   : 4;	// [0] record type
#endif
    } fat_b_rtype;
    union {
	struct {
#if !defined(__sparc)
	    byte_t  fat_m_fortrancc : 1;	// [1] fortran carriage control
	    byte_t  fat_m_impliedcc : 1;	// impiled carriage control
	    byte_t  fat_m_printcc   : 1;	// print file carriage control
	    byte_t  fat_m_nospan    : 1;	// do not span records
	    byte_t  fat_m_msbrcw    : 1;	// format of RCW
	    byte_t  fat_m_pad	    : 3;	// reserved
#else
	    byte_t  fat_m_pad	    : 3;	// reserved
	    byte_t  fat_m_msbrcw    : 1;	// format of RCW
	    byte_t  fat_m_nospan    : 1;	// do not span records
	    byte_t  fat_m_printcc   : 1;	// print file carriage control
	    byte_t  fat_m_impliedcc : 1;	// impiled carriage control
	    byte_t  fat_m_fortrancc : 1;	// [1] fortran carriage control
#endif
	} rattr;
	byte_t	attr;
    } fat_b_rattrib;
    ods_word_t	fat_w_rsize;	// [2] record size
    struct {
	ods_word_t	fat_w_hiblkh;	// [4] high word of the highest vb
	ods_word_t	fat_w_hiblkl;	// [6] low word of the hoghest vb
    } fat_l_hiblk;
    struct {
	ods_word_t	fat_w_efblkh;	// [8] high part of the eof blk
	ods_word_t	fat_w_efblkl;	// [10] low part of the eof blk
    } fat_l_efblk;
    ods_word_t	fat_w_ffbyte;	// [12] first free byte in the eof blk
    byte_t	fat_b_bktsize;	// [14] bucket size in blocks
    byte_t	fat_b_vfcsize;	// [15] size in bytes of fixed length control for VFC records
    ods_word_t	fat_w_maxrec;	// [16] maximum record size
    ods_word_t	fat_w_defext;	// [18] default extend quality
    ods_word_t	fat_w_gbc;	// [20]	global buffer count
    ods_word_t	fat_pad0[3];	// [22] reserved
    ods_word_t	fat_pad1;	// [28] not used
    ods_word_t	fat_w_versions;	// [30] def directory versions limit
};


//  File Header Structure

// 2.3.3.2 Ident Area
#define FH_NAME_SZ	20
#define FH_NAMEEX_SZ	66
#define FH_NAME_LENGTH  (FH_NAME_SZ + FH_NAMEEX_SZ + 1)

struct ODS_FI2 {
    byte_t	fi2_t_filename[20];	// [0] ASCII -> "NAME.TYPE;VERSION  "
    ods_word_t	fi2_w_revision;		// [20] revision number	
    ods_word_t	fi2_q_credate[4];	// [22] creation date and time
    ods_word_t	fi2_q_revdate[4];	// [30] revision date and time
    ods_word_t	fi2_q_expdate[4];	// [38] expiration date and time
    ods_word_t	fi2_q_backdate[4];	// [46] backup date and time
    byte_t	fi2_t_filenamext[66];	// [54] filename extension

public:
    //  filename processing
    //  obuf must be FH_NAME_LENGTH char buffer
    char*	fn2string(char* obuf) const;
    void	string2fn(const char* obuf);
};


// 2.3.3.3 Map Area

#define FM2_C_PLACEMENT	0x00
#define FM2_C_FORMAT1	0x01
#define FM2_C_FORMAT2	0x02
#define FM2_C_FORMAT3	0x03

struct ODS_FM2
{
    union {

	//  2.3.3.3.1 Retrieval Pointer Format 0
	struct {
#if !defined(__sparc)
		byte_t	fm2_v_exact	: 1;	// exect placement is required
		byte_t	fm2_v_oncyl	: 1;	// on cyl allocation
		byte_t	fm2_w_pad1	: 6;	// not used
#else
		byte_t	fm2_w_pad1	: 6;	// not used
		byte_t	fm2_v_oncyl	: 1;	// on cyl allocation
		byte_t	fm2_v_exact	: 1;	// exect placement is required
#endif
#if !defined(__sparc)
		byte_t	fm2_w_pad2	: 4;	// not used
		byte_t	fm2_v_lbn	: 1;	// alloc from next LBN
		byte_t	fm2_v_rvn	: 1;	// alloc on specified volume
		byte_t	fm2_v_format	: 2;	// format code
#else
		byte_t	fm2_v_format	: 2;	// format code
		byte_t	fm2_v_rvn	: 1;	// alloc on specified volume
		byte_t	fm2_v_lbn	: 1;	// alloc from next LBN
		byte_t	fm2_w_pad2	: 4;	// not used
#endif
	} ret_ptr0;

	//  2.3.3.3.2 Retrieval Pointer Format 1
	struct {
		byte_t	fm2_v_count1;		// [0] block count
#if !defined(__sparc)
		byte_t	fm2_v_highlbn	: 6;	// [1] high lbn bits
		byte_t	fm2_v_format	: 2;	// format spec
#else
		byte_t	fm2_v_format	: 2;	// format spec
		byte_t	fm2_v_highlbn	: 6;	// [1] high lbn bits
#endif
		byte_t	fm2_w_lowlbn[2];	// [2] low lbn bits
	} ret_ptr1;

	//  2.3.3.3.3 Retrieval Pointers Format 2
	struct {
		byte_t	fm2_v_count2_1	: 8;	// [0] block count
#if !defined(__sparc)
		byte_t	fm2_v_count2_2	: 6;	// [1] block count
		byte_t	fm2_v_format	: 2;	// format spec
#else
		byte_t	fm2_v_format	: 2;	// format spec
		byte_t	fm2_v_count2_2	: 6;	// [1] block count
#endif
		byte_t	fm2_l_lbn2[4];		// [2] lbn
	} ret_ptr2;

	//  2.3.3.3.4 Retrieval Pointer Format 3
	struct {
		byte_t	fm2_v_count2_1	: 8;	// [0] high 14 bits of the counter
#if !defined(__sparc)
		byte_t	fm2_v_count2_2	: 6;	// [1] high 14 bits of the counter
		byte_t	fm2_v_format	: 2;	// format spec
#else
		byte_t	fm2_v_format	: 2;	// format spec
		byte_t	fm2_v_count2_2	: 6;	// [1] high 14 bits of the counter
#endif
		byte_t	fm2_w_lowcount[2];	// [2] low 16 bits of the counter
		byte_t	fm2_l_lbn3[4];		// [4] lbn
	} ret_ptr3;

    };

public:
    int		rp_isfmt0() const;	// ret 1 if RP has format 0
    dword_t	rp_getwords() const;	// ret number of used words
    lbn_t	rp_getlbn() const;	// ret lbn (INV.. if RP0)
    dword_t	rp_getcount() const;	// ret blk count (start from 1)

    void	rp_setlbn(lbn_t lbn);
    void	rp_setcount(dword_t count);
    void	rp_init(int format, lbn_t lbn, dword_t count);
};

class rp_iterator
{
    ODS_FM2*	cur_rp;
    ODS_FM2*	end_rp;

public:

    rp_iterator(ODS_FM2* rp, int n) { 
	cur_rp = rp;
	end_rp = (ODS_FM2*)((word_t*)cur_rp + n); 
    }

    ODS_FM2*	operator++(int n) {
	ODS_FM2* ret_rp = cur_rp;
	word_t* next_rp = (word_t*)cur_rp + cur_rp->rp_getwords();
	return ((cur_rp < end_rp) ? (cur_rp = (ODS_FM2*)next_rp, ret_rp) : NULL);
    }
};

struct ODS_FH2 {
public:

    // 2.3.3.1 Header Area
    // number of field(s) actually used in a given instance depends on idoffset
    byte_t	fh2_b_idoffset;	    // [0] ident area offset
    byte_t	fh2_b_mpoffset;	    // [1] map area offset
    byte_t 	fh2_b_acoffset;	    // [2] access control list offset
    byte_t	fh2_b_rsoffset;	    // [3] reserved area offset
    ods_word_t	fh2_w_seg_num;	    // [4] segment number
    struct_lev	fh2_w_struclev;	    // [6] structure level and version (2,1)
    ODS_FID	fh2_w_fid;	    // [8] file identifier
    ODS_FID	fh2_w_ext_fid;	    // [14] extension file identifier
    ACP_FAT	fh2_w_recattr;	    // [20] record attributes
    ODS_FCH	fh2_l_filechar;	    // [52] file characteristics

    ods_word_t	fh2_pad0;	    // [56] reserved

    byte_t	fh2_b_map_inuse;    // [58] map words in use
    byte_t	fh2_b_acc_mode;	    // [59] accessor privilage level
    uic		fh2_l_fileowner;    // [60] UIC
    file_prot	fh2_w_fileprot;	    // [64] file protection code
    ODS_FID	fh2_w_backlink;	    // [66] back link file id
    byte_t	fh2_b_journal;	    // [72] journal control flags
    byte_t	fh2_b_ru_active;    // [73] recoverable facility ID number

    ods_word_t	fh2_pad1;	    // [74] reserved
    ods_dword_t	fh2_l_highwater;    // [76] highwater mark
    ods_word_t	fh2_pad2[4];	    // [80] reserved  
    ods_word_t	fh2_r_class_prot[10];// [88] sequrity classification mask


    //  Ident area isdescribed below

    //  Map area is described below

    //  Access control list area is not implemented

    //  User-Reserved area is not used by the Files-11 file system

    ods_word_t	pad3[201];	    // [108] some of the areas

    // End checksumm
    ods_word_t	fh2_w_checksum;	    // [510] end file header checksum

public:

    int		is_file_header(const ODS_FID* fid) const;
    int		is_file_contig() const;
    int		is_file_deleted() const;
    int		is_file_directory() const;

    int		has_valid_checksum() const;
    void	update_checksum();

    dword_t	get_hiblk() const;
    dword_t	get_eofblk() const;
    dword_t	get_file_length() const;
    word_t	get_ffbyte() const;
    dword_t	get_usedblk() const;

    void	set_hiblk(dword_t hiblk);
    void	set_eofblk(dword_t eofblk);
    void	set_ffbyte(word_t ffbyte);
    void    	set_file_length(dword_t length);

    dword_t	get_rp_blk_count() const;

    ODS_FI2*	get_id_area() const {
		    ASSERT(fh2_b_idoffset < fh2_b_mpoffset);
		    return (ODS_FI2*)((word_t*)this+fh2_b_idoffset);
		}
    ODS_FM2*	get_mp_area() const {
		    ASSERT(fh2_b_mpoffset < fh2_b_acoffset);
		    return (ODS_FM2*)((word_t*)this+fh2_b_mpoffset);
		}
};


// 2.4. Directory structures

#define DR_FID		0
#define DR_LINKNAME 	1
#define DR_MAX_FNAME	40

// Directory entry
struct ODS_dir_entry {
    ods_word_t	dir_w_version;		// [0] version of the directory entry
    ODS_FID	dir_w_fid;		// [2] file identifier
};

//  Directory record
struct ODS_dir_record {
    ods_word_t	dir_w_size;		// [0] record byte count
    ods_word_t	dir_w_verlimit;		// [2] file version limit
    struct {
#if !defined(__sparc)
	byte_t	dir_v_type 	 : 3;	// [4] type code
	byte_t	dir_b_flags_pad0 : 3;	// reserved
	byte_t	nextrec		 : 1;	// nextrec has same name & type
	byte_t	prevrec		 : 1;	// prevrec has same name & type
#else
	byte_t	prevrec		 : 1;	// prevrec has same name & type
	byte_t	nextrec		 : 1;	// nextrec has same name & type
	byte_t	dir_b_flags_pad0 : 3;	// reserved
	byte_t	dir_v_type 	 : 3;	// [4] type code
#endif
    } dir_b_flags;
    byte_t	dir_b_namecount;	// [5] file name length
    byte_t	dir_t_name[2];		// [6] file name in format NAME.TYPE
					// max FNAME == 40, if strlen(name)
					// is odd '\0' is padded
public:

    dword_t	get_entry_count() const;
    ODS_dir_entry* 
		get_entry(dword_t numb) const;
    int		is_nullsize_record() const {
		    return (dir_w_size == 0x0000);
		}
    int		is_skip_record() const {
		    return (dir_w_size == 0xFFFF);
		}
    int		is_empty_record() const {
		    return (is_nullsize_record() || is_skip_record());
		}
};

//  Storage Control Block
struct ODS_SCB {
    struct_lev	scb_w_struclev;		// [0]
    ods_word_t	scb_w_cluster;		// [2]
    ods_dword_t	scb_l_volsize;		// [4]
    ods_dword_t	scb_l_blksize;		// [8]
    ods_dword_t	scb_l_sectors;		// [12]
    ods_dword_t	scb_l_tracks;		// [16]
    ods_dword_t	scb_l_cylinder;		// [20]
    ods_word_t	scb_pad[243];		// [24]
    ods_word_t	scb_w_checksum;		// [510] End checksum

public:
    int		is_valid() const;
    int		has_valid_checksum() const;
    void	update_checksum();

    dword_t	get_bitmap_size();
};

#endif
